#include <user_code.hpp>

// AC Control Locking
char AC_LOCK_TOPIC[75];
#define AC_LOCK_ADDRESS 15002
bool ac_lock = false;

// Display Componets
NexText clock_txt = NexText(1, 2, "clock");
NexDSButton lt_bt = NexDSButton(1, 3, "lt_bt");
NexDSButton fan_bt = NexDSButton(1, 4, "fan_bt");
NexDSButton puri_bt = NexDSButton(1, 5, "puri_bt");
NexButton up_bt = NexButton(1, 6, "up_bt");
NexButton down_bt = NexButton(1, 7, "down_bt");
NexButton mode_off_btn = NexButton(1, 8, "mode_off_btn");
NexButton mode_fan_btn = NexButton(1, 9, "mode_fan_btn");
NexButton mode_cool_btn = NexButton(1, 10, "mode_cool_btn");
NexButton fan_auto_btn = NexButton(1, 11, "fan_auto_btn");
NexButton fan_1_btn = NexButton(1, 12, "fan_1_btn");
NexButton fan_2_btn = NexButton(1, 13, "fan_2_btn");
NexButton fan_3_btn = NexButton(1, 14, "fan_3_btn");
NexText temp_txt = NexText(1, 15, "temp_txt");

// List of Component ID Message to listen to
NexTouch *nex_listen_list[] =
    {
        &lt_bt,
        &fan_bt,
        &puri_bt,
        &up_bt,
        &down_bt,
        &mode_off_btn,
        &mode_fan_btn,
        &mode_cool_btn,
        &fan_auto_btn,
        &fan_1_btn,
        &fan_2_btn,
        &fan_3_btn,
        NULL};

/*
This Code will run right after ESPMega PRO's
Peripheral Initialization Routine
*/
void user_pre_init()
{
    nexInit();
}

/*
This code will run after every component is initialized
*/
void user_init()
{
    ESPMega_EXTLCD.print("page dashboard");
    ESPMega_EXTLCD.write(0xFF);
    ESPMega_EXTLCD.write(0xFF);
    ESPMega_EXTLCD.write(0xFF);
    memcpy(AC_LOCK_TOPIC, MQTT_BASE_TOPIC, 20);
    strcat(AC_LOCK_TOPIC, "/ac/lock");
    ac_lock = ESPMega_FRAM.read8(AC_LOCK_ADDRESS);
    ac_lock = ac_lock > 1 ? 0 : ac_lock;
    ESPMega_EXTLCD.print("dashboard.pic=");
    ESPMega_EXTLCD.print(ac_lock ? "2" : "1");
    ESPMega_EXTLCD.write(0xFF);
    ESPMega_EXTLCD.write(0xFF);
    ESPMega_EXTLCD.write(0xFF);
    mqtt.subscribe(AC_LOCK_TOPIC, ac_lock_callback);
    lt_bt.attachPop(lt_btn_cb, &lt_bt);
    fan_bt.attachPop(fan_btn_cb, &fan_bt);
    puri_bt.attachPop(puri_btn_cb, &puri_bt);
    up_bt.attachPop(temp_up_btn_cb, &up_bt);
    down_bt.attachPop(temp_down_btn_cb, &down_bt);
    mode_off_btn.attachPop(mode_off_btn_cb, &mode_off_btn);
    mode_fan_btn.attachPop(mode_fan_btn_cb, &mode_fan_btn);
    mode_cool_btn.attachPop(mode_cool_btn_cb, &mode_cool_btn);
    fan_auto_btn.attachPop(fan_auto_btn_cb, &fan_auto_btn);
    fan_1_btn.attachPop(fan_1_btn_cb, &fan_1_btn);
    fan_2_btn.attachPop(fan_2_btn_cb, &fan_2_btn);
    fan_3_btn.attachPop(fan_3_btn_cb, &fan_3_btn);
}

/*
This code will run once every event loop
*/
void user_loop()
{
    nexLoop(nex_listen_list);
}

/*
This code will run when an input pin changed state
*/
void virtual_interrupt_user_callback(int pin, int state)
{
    if (pin >= 0 & pin <= 7)
    {
        pwm_toggle(pin);
    }
}

void pwm_changed_user_callback(int pin)
{
    if ((pin == LIGHT1_PIN) || (pin == LIGHT2_PIN) || (pin == LIGHT3_PIN) || (pin = LIGHT4_PIN))
    {
        lt_bt.setValue(cud_light_group_state());
    }
    else if (pin == FAN1_PIN || pin == FAN2_PIN || pin == FAN3_PIN)
    {
        fan_bt.setValue(cud_fan_group_state());
    }
    else if (pin == AIR_PURIFIER_PIN)
    {
        puri_bt.setValue(pwm_get_state(pin));
    }
}

/*
This code will run every 15 seconds
*/
void timer_tick_callback()
{
    char time_buffer[15];
    rtctime_t time = ESPMega_getTime();
    sprintf(time_buffer, "%02d:%02d", time.hours, time.minutes);
    clock_txt.setText(time_buffer);
}

void lt_btn_cb(void *comp)
{
    cud_light_toggle();
}
void fan_btn_cb(void *comp)
{
    cud_fan_toggle();
}
void puri_btn_cb(void *comp)
{
    pwm_toggle(AIR_PURIFIER_PIN);
}
void temp_up_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        uint8_t new_temp = ac_get_temperature();
        ac_set_state(ac_get_mode(), ac_get_temperature() + 1, ac_get_fan_speed());
    }
}
void temp_down_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        uint8_t new_temp = ac_get_temperature();
        ac_set_state(ac_get_mode(), ac_get_temperature() - 1, ac_get_fan_speed());
    }
}
void mode_cool_btn_cb(void *comp)
{
    if (!ac_lock)
        ac_set_state(1, ac_get_temperature(), ac_get_fan_speed());
}
void mode_fan_btn_cb(void *comp)
{
    if (!ac_lock)
        ac_set_state(2, ac_get_temperature(), ac_get_fan_speed());
}
void mode_off_btn_cb(void *comp)
{
    if (!ac_lock)
        ac_set_state(0, ac_get_temperature(), ac_get_fan_speed());
}

void fan_auto_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        ac_set_state(ac_get_mode(), ac_get_temperature(), 0);
    }
}
void fan_1_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        ac_set_state(ac_get_mode(), ac_get_temperature(), 3);
    }
}
void fan_2_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        ac_set_state(ac_get_mode(), ac_get_temperature(), 2);
    }
}
void fan_3_btn_cb(void *comp)
{
    if (!ac_lock)
    {
        ac_set_state(ac_get_mode(), ac_get_temperature(), 1);
    }
}

void cud_light_toggle()
{
    bool new_state = !cud_light_group_state();
    pwm_set_state(LIGHT1_PIN, new_state);
    pwm_set_state(LIGHT2_PIN, new_state);
    pwm_set_state(LIGHT3_PIN, new_state);
    pwm_set_state(LIGHT4_PIN, new_state);
}
bool cud_light_group_state()
{
    return pwm_get_state(LIGHT1_PIN) || pwm_get_state(LIGHT2_PIN) || pwm_get_state(LIGHT3_PIN) || pwm_get_state(LIGHT4_PIN);
}
void cud_fan_toggle()
{
    bool new_state = !cud_fan_group_state();
    pwm_set_state(FAN1_PIN, new_state);
    pwm_set_state(FAN2_PIN, new_state);
    pwm_set_state(FAN3_PIN, new_state);
}
bool cud_fan_group_state()
{
    return pwm_get_state(FAN1_PIN) || pwm_get_state(FAN2_PIN) || pwm_get_state(FAN3_PIN);
}

void ac_changed_user_callback(int mode, int temperature, int fan_speed)
{
    char temp_text[3];
    sprintf(temp_text, "%dC", temperature);
    temp_txt.setText(temp_text);
    mode_off_btn.Set_background_image_pic(mode == 0 ? 14 : 13);
    mode_fan_btn.Set_background_image_pic(mode == 2 ? 16 : 15);
    mode_cool_btn.Set_background_image_pic(mode == 1 ? 18 : 17);
    fan_auto_btn.Set_background_image_pic(fan_speed == 0 ? 20 : 19);
    fan_1_btn.Set_background_image_pic(fan_speed == 3 ? 22 : 21);
    fan_2_btn.Set_background_image_pic(fan_speed == 2 ? 24 : 23);
    fan_3_btn.Set_background_image_pic(fan_speed == 1 ? 26 : 25);
}

void ac_lock_callback(String topic, String payload)
{
    if (payload.equals("lock"))
    {
        ac_lock = true;
        ESPMega_FRAM.write8(AC_LOCK_ADDRESS, ac_lock);
        ESPMega_EXTLCD.print("dashboard.pic=");
        ESPMega_EXTLCD.print(ac_lock ? "2" : "1");
        ESPMega_EXTLCD.write(0xFF);
        ESPMega_EXTLCD.write(0xFF);
        ESPMega_EXTLCD.write(0xFF);
    }
    else if (payload.equals("unlock"))
    {
        ac_lock = false;
        ESPMega_FRAM.write8(AC_LOCK_ADDRESS, ac_lock);
        ESPMega_EXTLCD.print("dashboard.pic=");
        ESPMega_EXTLCD.print(ac_lock ? "2" : "1");
        ESPMega_EXTLCD.write(0xFF);
        ESPMega_EXTLCD.write(0xFF);
        ESPMega_EXTLCD.write(0xFF);
    }
}